#ifndef USOCKET_H
#define USOCKET_H

#include "ra_types.h"

typedef enum
{
    USOCKET_TYPE_NONE,
    USOCKET_TYPE_PLAIN,
} usocket_type_t;

struct usocket
{
    usocket_type_t type;
    union
    {
        struct
        {
            int fd;
        } as_plain;
    } u;
};
typedef struct usocket usocket_t;

/* Initialize usocket */
void usocket_init( \
        usocket_t *usocket);

/* Connect to a plain host
 * @return 0  success
 * @return -1 fail */
int usocket_connect3( \
        usocket_t *usocket, \
        const char *host, const ra_size_t port, const int timeout, int blocking);

/* Connect to a plain host
 * @return 0  success
 * @return -1 fail */
int usocket_connect2( \
        usocket_t *usocket, \
        const char *host, const ra_size_t port, const int timeout);

/* Connect to a plain host
 * @return 0  success
 * @return -1 fail */
int usocket_connect( \
        usocket_t *usocket, \
        const char *host, const ra_size_t port);

/* Test if the connection is established. */ 
int usocket_alive( \
        usocket_t *usocket);

/* Close
 * Disconnect the connection. */
void usocket_close( \
        usocket_t *usocket);

/* Readable,
 * For Plain socket. */
int usocket_readable( \
        usocket_t *usocket);

/* Recv */
int usocket_recv( \
        usocket_t *usocket, \
        void *buf, ra_size_t len);

/* Send */
int usocket_send( \
        usocket_t *usocket, \
        const void *buf, ra_size_t len);


#endif

